
# Parsers.py
#
# $Id: parsers.py,v 1.1 2006/11/09 14:28:41 kevinadmin Exp $
#
# Very simple parsers for different types of sources
#
# Each return two lists, Called methods and resources
#
# The mapping of object meta_types is done through the 
# PARSERS dictionary at the bottom of this file.
#
# The parsers are very trivial. In particular, they cannot
# handle getattr and references from objects. This is why
# they are static. There is other product(s) which use
# the profiler output to generate a tree. Check zope.org.

import string

# I am interested in objects that impact on the acl_users.
# this is login, logout and create users etc.
PYTHONRESOURCENAMES=['acl_users']

# How to find the end of an identifier
PYTHONIDENTIFIERALLOWED=string.ascii_letters+string.digits+'_ .\t'

# These container references are not acquisitons. 
IGNORE=['absolute_url', 'REQUEST']

# These are objects that we have no intention of scanning.
# have a list of them so that the caller can have better
# error messaging.
NONEXECUTABLEOBJECTS=['Folder', 'File', 'Zope Static Call Tree', 
	'Temporary Folder', 'Transient Object Container',
	'Session Data Manager', 'Site Error Log',
	'Virtual Host Monster', 'Browser Id Manager', 'Help Topic',
	'Product Help', 'User Folder', 'Control Panel',
	'Database Management', 'Version Management', 'WebDAV Lock Manager',
	'Product Management', 'Product', 'Debug Information',
        'Accelerated HTTP Cache Manager', 'External Method', 
        'Image', 'Folder (Ordered)', 'Mail Host',
	'Z Psycopg Database Connection', 'Authorisation Source',
	'ZWiki Page', 'Zwiki Outline Cache',
]

# My Local Objects
NONEXECUTABLEOBJECTS=NONEXECUTABLEOBJECTS+ [
	'NAUtility', 'NACompanyAdvert', 'NATextMessage', 'TQPollingThread',
]

def PythonParser(obj):
    """Parse python source code.

    The only resource that I am interested in is acl_users
    """
    if type(obj) == type(''): s = obj
    else: s = obj.document_src()

    methods=[]
    resources=[]
    for resource in PYTHONRESOURCENAMES:
        if s.find(resource) != -1:
            resources.append(resource)

    # Simply wind through the source code string.
    offset = s.find('container.')
    while offset != -1:
        start=offset
        for end in range(offset+10, len(s)):
            if s[end] not in PYTHONIDENTIFIERALLOWED:
                break
        method = s[offset+10:end].strip()
        method=method.replace('.', '/')  # python paths to urls
        for ign in IGNORE:
            if method[:len(ign)] == ign:
                method=None
                break

        if method and method not in methods:
            methods.append(method)
        offset = s.find('container.', end)
    return (methods, resources)

def ZPTParser(obj):
    """Parse ZPT .

    """
    if type(obj) == type(''): s = obj
    else: s = obj.document_src()

    methods=[]
    resources=[]

    # Simply wind through the source code string.
    offset1 = s.find('container.')
    offset2 = s.find('container/')
    if offset1 >=0 and offset2 == -1: offset=offset1
    elif offset1 >=0 and offset1 < offset2: offset=offset1
    else: offset = offset2

    if offset > 0:
        if s[offset+9] == '/':
	    use_slash=True
        else:
	    use_slash=False

    while offset != -1:
        start=offset
        for end in range(offset+10, len(s)):
            if s[end] in ['/']:
                if not use_slash: break
            elif s[end] not in PYTHONIDENTIFIERALLOWED:
                break

        method = s[offset+10:end].strip()

        if not use_slash:
            method=method.replace('.', '/')  # python paths to urls

        if method and method not in methods:
            methods.append(method)

        offset1 = s.find('container.', end)
        offset2 = s.find('container/', end)
        if offset1 >=0 and offset2 == -1: offset=offset1
        elif offset1 >=0 and offset1 < offset2: offset=offset1
        else: offset = offset2
        if offset > 0:
            if s[offset+9] == '/':
	        use_slash=True
            else:
	        use_slash=False

    return (methods, resources)

def ZSQLParser(obj):
    """All I want to do is find the table name. This is prefixed by
    from (select x from, delete from), update, into
    """

    if type(obj) == type(''): s = obj
    else: s = obj.document_src()

    s=s.lower()

    offset = s.find('</params>') + 9
    s=s[offset:]
    if s[0] in ['\n', '\r']: s=s[1:]
    if s[0] in ['\n', '\r']: s=s[1:]
    if s[0] in ['\n', '\r']: s=s[1:]
    if s[0] in ['\n', '\r']: s=s[1:]
    if s[0] in ['\n', '\r']: s=s[1:]
    if s[0] in ['\n', '\r']: s=s[1:]

    leadword=''
    table = ''
    command=s.split()[0]
    if command in ['insert']:
        leadword = 'into'
    elif command in ['select', 'delete']:
        leadword = 'from'
    elif command in ['update']:
        leadword = 'update'

    if leadword:
        offset = s.find(leadword)
        if offset != -1:
            table = s[offset+len(leadword):].strip().split()[0]
    return ([], ['SQL:%s:%s' % (command, table)])

PARSERS={
	'Page Template': ZPTParser,
	'Script (Python)': PythonParser,
	'Z SQL Method': ZSQLParser,
}

# Local Products
PARSERS['NA PublicPage']= ZPTParser


